﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace Shaed.Scenes
{
    class Platformer : Scene
    {
        Textbox textHealth;

        // Constructor
        public Platformer()
        {
            name = "Platformer";
            bounds = new Rectangle(0, 0, 2048, 1024);
            map = new TileMap(bounds.Width, bounds.Height, 16);
            gameObjects = new List<GameObject>();
            backgroundColor = Color.DarkSlateGray;
            camera = new Camera(this);
            camera.Zoom(1);
        }

        // Load
        public override void Load()
        {
            // Place Tiles


            map.AddTileType("Grass",                Assets.tiles, 2, 8, 16, false, "b4be8c", 0.01f);
            map.AddTileType("Grass Roots",          Assets.tiles, 2, 9, 16, true, "dcd2c8", 0);
            map.AddTileType("Dirt",                 Assets.tiles, 2, 10, 16, true, "64503c", 0);
            map.AddTileType("Dirt Bottom",          Assets.tiles, 2, 11, 16, false, "503e2a", 0);
            map.AddTileType("Purp Brick 1",         Assets.tiles, 3, 9, 16, true, "b9a5a5", 0);
            map.AddTileType("Purp Brick 2",         Assets.tiles, 4, 9, 16, true, "917d7d", 0);
            map.AddTileType("Purp Brick 3",         Assets.tiles, 5, 9, 16, true, "a59191", 0);
            map.AddTileType("Purp Pillar 1",        Assets.tiles, 3, 10, 16, false, "b985a5", 1.0f);
            map.AddTileType("Purp Pillar 2",        Assets.tiles, 3, 11, 16, false, "9e6c8b", 1.0f);
            map.AddTileType("Purp Pillar 3",        Assets.tiles, 3, 12, 16, false, "865674", 1.0f);
            map.AddTileType("Checker Top 1",        Assets.tiles, 6, 10, 16, true, "393939", 0);
            map.AddTileType("Checker Top 2",        Assets.tiles, 7, 10, 16, true, "484848", 0);
            map.AddTileType("Checker Bottom 1",     Assets.tiles, 6, 11, 16, true, "565656", 0);
            map.AddTileType("Checker Bottom 2",     Assets.tiles, 7, 11, 16, true, "656565", 0);

            map.CreateMapFromTexture(Assets.platformerLevel01);

            // Place GameObjects
            foreach (Vector2 position in map.GetPositionsByColor(Assets.platformerLevel01, "f95151"))
            {
                AddGameObject(new Objects.Ball(), this, (int) position.X, (int) position.Y);
            }
            foreach (Vector2 position in map.GetPositionsByColor(Assets.platformerLevel01, "353535"))
            {
                AddGameObject(new Objects.Powerup(), this, (int)position.X, (int)position.Y);
            }
            foreach (Vector2 position in map.GetPositionsByColor(Assets.platformerLevel01, "6f6f6f"))
            {
                AddGameObject(new Objects.Brick(), this, (int)position.X, (int)position.Y);
            }

            // Textboxes
            textHealth = new Textbox(Assets.font, 10, 10, "HEALTH: 100");
            textHealth.scale = 1.5f;
            textboxes.Add(textHealth);

            // Create Player
            player = new Entities.Wraith();
            AddGameObject(player, this, 64, 900);
            camera.parent = player.transform;
        }

        // Update
        public override void Update(GameTime gameTime)
        {
            // Controls
            if (Controller.IsKeyDown(Keys.Escape)) // Exit game
            {
                Program.game.Exit();
            }

            // Camera Follow Player
            camera.Update();
        }
    }
}
